@@ -28,6 +28,10 @@
  * asynchronous API and threaded mainloop.
  * Eric Thornton, KM4DSJ 2015
 */
+/*
+ * 2017 Nov by N2ADR Remove most exit(1) statements.
+ * 2017 Nov by N2ADR Add code to enable PulseAudio to continue with mis-spelled device names.
+*/
 
 #include <Python.h>
 #include <stdio.h>
@@ -39,13 +43,6 @@
 #include "quisk.h"
 #include <pulse/pulseaudio.h>
 
-//compiling with verbose = 1 shows stream connection info.
-//compiling with verbose = 2 shows stream latency on console @ TIME_EVENT_USEC frequency
-int verbose = 0;
-
-//This is for verbose = 2 latency measurements
-#define TIME_EVENT_USEC 100000
-
 // From pulsecore/macro.h
 #define pa_memzero(x,l) (memset((x), 0, (l)))
 #define pa_zero(x) (pa_memzero(&(x), sizeof(x)))
@@ -58,18 +55,14 @@ static pa_threaded_mainloop *pa_ml;
 static pa_mainloop_api *pa_mlapi;
 static pa_context *pa_ctx; 		//local context
 static pa_context *pa_IQ_ctx; 		//remote context for IQ audio
-static pa_time_event *time_event_local;
-static pa_time_event *time_event_IQ;
 volatile int streams_ready = 0;		//This is ++/-- by the mainloop thread
+volatile int streams_to_start;
 
 // remember all open devices for easy cleanup on exit
-static pa_stream* OpenPulseDevices[PA_LIST_SIZE * 2] = {NULL};
-
-//sorted lists of local and remote devices
-static struct sound_dev *LocalPulseDevices[PA_LIST_SIZE] = {NULL};
-static struct sound_dev *RemotePulseDevices[PA_LIST_SIZE] = {NULL};
-
+static pa_stream *OpenPulseDevices[PA_LIST_SIZE * 2] = {NULL};
 
+static int have_QuiskDigitalInput;	// Do we need to create this null sink?
+static int have_QuiskDigitalOutput;	// Do we need to create this null sink?
 
 /* This callback happens any time a stream changes state. Here, it's primary used to 
  * tell the quisk thread when streams are ready. 
@@ -80,19 +73,25 @@ void stream_state_callback(pa_stream *s, void *userdata) {
     assert(s);
     assert(dev);
     
-    switch (pa_stream_get_state(s)) {
+    dev->pulse_stream_state = pa_stream_get_state(s);
+    switch (dev->pulse_stream_state) {
         case PA_STREAM_CREATING:
+	    if (quisk_sound_state.verbose_pulse)
+		printf("\n**Stream %s state Creating\n", dev->name);
             break;
             
         case PA_STREAM_TERMINATED:
-            if (verbose)
-                printf("stream %s terminated\n", dev->name);
+	    if (quisk_sound_state.verbose_pulse)
+		printf("\n**Stream %s state Terminated\n", dev->name);
             streams_ready--;
             break;
             
         case PA_STREAM_READY:
+	    if (quisk_sound_state.verbose_pulse)
+		printf("\n**Stream %s state Ready\n", dev->name);
             streams_ready++; //increment counter to tell other thread that this stream is ready
-            if (verbose) {
+	    streams_to_start++;
+            if (quisk_sound_state.verbose_pulse) {
                 const pa_buffer_attr *a;
                 printf("Connected to device %s (%u, %ssuspended). ",
                        pa_stream_get_device_name(s), pa_stream_get_device_index(s),
@@ -112,8 +111,13 @@ void stream_state_callback(pa_stream *s, void *userdata) {
         
         case PA_STREAM_FAILED:
         default:
-            printf("Stream error: %s - %s\n", dev->name, pa_strerror(pa_context_errno(pa_stream_get_context(s))));
-            exit(1);
+	    snprintf(quisk_sound_state.err_msg, QUISK_SC_SIZE,
+		"Stream error: %s - %s", dev->name, pa_strerror(pa_context_errno(pa_stream_get_context(s))));
+	    if (quisk_sound_state.verbose_pulse)
+		printf("\n**Stream %s state Failed\n", dev->name);
+            printf("%s\n", quisk_sound_state.err_msg);
+	    streams_to_start++;
+            break;
     }
 }
 
@@ -124,7 +128,7 @@ static void stream_underflow_callback(pa_stream *s, void *userdata) {
     assert(s);
     assert(dev);
     
-    if (verbose)
+    if (quisk_sound_state.verbose_pulse)
         printf("Stream underrun %s\n", dev->name);
     dev->dev_error++;
 }
@@ -134,7 +138,7 @@ static void stream_overflow_callback(pa_stream *s, void *userdata) {
     struct sound_dev *dev = userdata;
     assert(s);
     
-    if (verbose)
+    if (quisk_sound_state.verbose_pulse)
         printf("Stream overrun %s\n", dev->name);
     dev->dev_error++;
 }
@@ -145,7 +149,7 @@ static void stream_started_callback(pa_stream *s, void *userdata) {
     assert(s);
     assert(dev);
     
-    if (verbose)
+    if (quisk_sound_state.verbose_pulse)
         printf("Stream started %s\n", dev->name);
 }
 
@@ -155,14 +159,14 @@ static void stream_corked_callback(pa_stream *s, int success, void *userdata) {
     struct sound_dev *dev = userdata;
     
     if (s) {
-        if (verbose)
+        if (quisk_sound_state.verbose_pulse)
             printf("Stream cork/uncork %s success\n", dev->name);
         pa_threaded_mainloop_signal(pa_ml, 0);
     }
     else {
-        if (verbose)
+        if (quisk_sound_state.verbose_pulse)
             printf("Stream cork/uncork %s Failure!\n", dev->name);
-        exit(1);
+        exit(11);
     }
 }
 
@@ -177,7 +181,7 @@ static void stream_flushed_callback(pa_stream *s, int success, void *userdata) {
     }
     else {
         printf("Stream flush %s Failure!\n", dev->name);
-        exit(1);
+        exit(12);
     }
 }
 
@@ -200,51 +204,6 @@ static void stream_timing_callback(pa_stream *s, int success, void *userdata) {
 }
 
 
-
-// Show the current latency on the console for debugging purposes
-static void stream_debug_timing_callback(pa_stream *s, int success, void *userdata) {
-    struct sound_dev *dev = userdata;
-    pa_usec_t l;
-    int negative = 0;
-    assert(s);
-
-    if (!success || pa_stream_get_latency(s, &l, &negative) < 0) {
-        printf("pa_stream_get_latency() failed: %s\n", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
-        return;
-    }
-
-    printf("%s Latency:%7.0fus", dev->stream_description, (float)(l)*(negative?-1.0f:1.0f));
-}
-
-//Called on frequency determined by TIME_EVENT_USEC
-static void time_event_callback(pa_mainloop_api *m, pa_time_event *e, const struct timeval *t, void *userdata) {
-    assert(userdata);
-    struct sound_dev **pDevices = userdata; 
-
-    while(*pDevices) {
-        struct sound_dev *dev = *pDevices++;
-        assert(dev);
-        pa_stream *s = dev->handle;
-        
-        if (s && pa_stream_get_state(s) == PA_STREAM_READY) {
-            pa_operation *o;
-            if (!(o = pa_stream_update_timing_info(s, stream_debug_timing_callback, dev)))
-                printf("pa_stream_update_timing_info() failed: %s\n", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
-            else
-                pa_operation_unref(o);
-        }
-    }
-
-    printf("\r");
-    fflush(stdout);
-    
-    if (e == time_event_IQ)
-        pa_context_rttime_restart(pa_IQ_ctx, e, pa_rtclock_now() + TIME_EVENT_USEC);
-    else
-        pa_context_rttime_restart(pa_ctx, e, pa_rtclock_now() + TIME_EVENT_USEC);
-}
-
-
 /* This callback allows us to read in the server side stream information so that we
  * can match stream formats and sizes to what is configured in pulseaudio.
 */
@@ -274,8 +233,8 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
         else
             dev_name = NULL;			// the device name is "pulse" for the default device
 
-        if (verbose)
-            printf("Opening Device %s ", dev_name);
+        if (quisk_sound_state.verbose_pulse)
+            printf("Opening Device %s\n", dev_name);
 
         //Construct sample specification. Use S16LE if availiable. Default to Float32 for others.
         //If the source/sink is not Float32, Pulseaudio will convert it (uses CPU)
@@ -309,11 +268,11 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
 
             if (!(s = pa_stream_new(c, dev->stream_description, &ss, NULL))) {
                 printf("pa_stream_new() failed: %s", pa_strerror(pa_context_errno(c)));
-                exit(1);
+                continue;
             }
             if (pa_stream_connect_record(s, dev_name, &rec_ba, rec_flags) < 0) {
                 printf("pa_stream_connect_record() failed: %s", pa_strerror(pa_context_errno(c)));
-                exit(1);
+                continue;
             }
             pa_stream_set_overflow_callback(s, stream_overflow_callback, dev);
         }
@@ -324,11 +283,11 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
 
             if (!(s = pa_stream_new(c, dev->stream_description, &ss, NULL))) {
                 printf("pa_stream_new() failed: %s", pa_strerror(pa_context_errno(c)));
-                exit(1);
+                continue;
             }
             if (pa_stream_connect_playback(s, dev_name, &play_ba, pb_flags, pa_cvolume_set(&cv, ss.channels, volume), NULL) < 0) {
                 printf("pa_stream_connect_playback() failed: %s", pa_strerror(pa_context_errno(c)));
-                exit(1);
+                continue;
             }
             pa_stream_set_underflow_callback(s, stream_underflow_callback, dev);
         }
@@ -387,7 +346,7 @@ static void stream_drain_complete(pa_stream*s, int success, void *userdata) {
         quit(1);
     }
 
-    if (verbose)
+    if (quisk_sound_state.verbose_pulse)
         printf("Playback stream %s drained.\n", dev->name);
 }
 
@@ -416,7 +375,7 @@ void quisk_cork_pulseaudio(struct sound_dev *dev, int b) {
     
     if (!(o = pa_stream_cork(s, b, stream_corked_callback, dev))) {
         printf("pa_stream_cork(): %s\n", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
-        exit(1);
+        exit(13);
     }
     else {
         while(pa_operation_get_state(o) == PA_OPERATION_RUNNING)
@@ -441,7 +400,7 @@ void quisk_flush_pulseaudio(struct sound_dev *dev) {
     
     if (!(o = pa_stream_flush(s, stream_flushed_callback, dev))) {
         printf("pa_stream_flush(): %s\n", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
-        exit(1);
+        exit(14);
     }
     else {
         while(pa_operation_get_state(o) == PA_OPERATION_RUNNING)
@@ -477,12 +436,15 @@ int quisk_read_pulseaudio(struct sound_dev *dev, complex double *cSamples) {
     pa_stream *s = dev->handle;
     size_t read_bytes;
     
-    if (!dev)
-        return 0;
-    
-    if (dev->cork_status)
+    if (!dev || dev->pulse_stream_state != PA_STREAM_READY)
         return 0;
 
+    if (dev->cork_status) {
+        if (dev->read_frames != 0) {
+            WaitForPoll();
+        }
+        return 0;
+    }
 
    // Note: Access to PulseAudio data from our sound thread requires locking the threaded mainloop.
     if (dev->read_frames == 0) {		// non-blocking: read available frames
@@ -507,7 +469,6 @@ int quisk_read_pulseaudio(struct sound_dev *dev, complex double *cSamples) {
             return 0;
         }
         dev->dev_latency = read_frames * dev->num_channels * 1000 / (dev->sample_rate / 1000);
-        read_frames = dev->read_frames;
     }
     
     
@@ -530,7 +491,7 @@ int quisk_read_pulseaudio(struct sound_dev *dev, complex double *cSamples) {
        }
        
        if (nSamples * dev->num_channels * dev->sample_bytes + read_bytes >= SAMP_BUFFER_SIZE) {
-           if (verbose)
+           if (quisk_sound_state.verbose_pulse)
                printf("buffer full on %s\n", dev->name);
            pa_stream_drop(s);		// limit read request to buffer size
            break;
@@ -598,7 +559,7 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
     void *fbuffer;
     int fbuffer_bytes = 0;
     
-    if( !dev || nSamples <= 0)
+    if( !dev || nSamples <= 0 || dev->pulse_stream_state != PA_STREAM_READY)
         return;
     
     if (dev->cork_status)
@@ -649,7 +610,7 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
 
     else {
        printf("Unknown sample size for %s", dev->name);
-       exit(1);
+       exit(15);
     }
 
 
@@ -663,7 +624,7 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
         if ( writable > 1024*1000 ) //sanity check to prevent pa_xmalloc from crashing on monitor streams
             writable = 1024*1000;
         if (fbuffer_bytes > writable) {
-            if (verbose)
+            if (quisk_sound_state.verbose_pulse)
                 printf("Truncating write by %u bytes\n", fbuffer_bytes - (int)writable);
             fbuffer_bytes = writable;
         }
@@ -671,7 +632,7 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
         //printf("wrote %d to %s\n", writable, dev->name);
     }
     else {
-        if (verbose)
+        if (quisk_sound_state.verbose_pulse)
             printf("Can't write to stream %s. Dropping %d bytes\n", dev->name, fbuffer_bytes);
     }
     
@@ -687,6 +648,7 @@ void sort_devices(struct sound_dev **plist, struct sound_dev **pLocal, struct so
     while(*plist) {
         struct sound_dev *dev = *plist++;
 
+	dev->pulse_stream_state = PA_STREAM_UNCONNECTED;
         // Not a PulseAudio device
         if( dev->driver != DEV_DRIVER_PULSEAUDIO )
             continue;
@@ -737,9 +699,24 @@ void sort_devices(struct sound_dev **plist, struct sound_dev **pLocal, struct so
 void quisk_start_sound_pulseaudio(struct sound_dev **pCapture, struct sound_dev **pPlayback) {
     int num_pa_devices = 0;
     int i;
+    //sorted lists of local and remote devices
+    struct sound_dev *LocalPulseDevices[PA_LIST_SIZE] = {NULL};
+    struct sound_dev *RemotePulseDevices[PA_LIST_SIZE] = {NULL};
 
     sort_devices(pCapture, LocalPulseDevices, RemotePulseDevices);
     sort_devices(pPlayback, LocalPulseDevices, RemotePulseDevices);
+    pa_IQ_ctx = NULL;
+    pa_ctx = NULL;
+    pa_ml = NULL;
+    pa_mlapi = NULL;
+    streams_to_start = 0;
+    //quisk_sound_state.verbose_pulse = 1;
+    
+    if (!RemotePulseDevices[0] && !LocalPulseDevices[0]) {
+        if (quisk_sound_state.verbose_pulse)
+            printf("No pulseaudio devices to open!\n");
+        return; //nothing to open. No need to start the mainloop.
+    }
 
     // Create a mainloop API
     pa_ml = pa_threaded_mainloop_new();
@@ -749,7 +726,7 @@ void quisk_start_sound_pulseaudio(struct sound_dev **pCapture, struct sound_dev
   
     if (pa_threaded_mainloop_start(pa_ml) < 0) {
         printf("pa_mainloop_run() failed.");
-        exit(1);
+	return;
     }
     else
         printf("Pulseaudio threaded mainloop started\n");
@@ -770,15 +747,6 @@ void quisk_start_sound_pulseaudio(struct sound_dev **pCapture, struct sound_dev
         pa_context_set_state_callback(pa_ctx, state_cb, LocalPulseDevices);
     }
 
-    if (!RemotePulseDevices[0] && !LocalPulseDevices[0]) {	//no pulseaudio devices to open!
-        if (verbose)
-            printf("No pulseaudio devices to open!\n");
-        pa_threaded_mainloop_unlock(pa_ml);
-        pa_threaded_mainloop_stop(pa_ml);
-        pa_threaded_mainloop_free(pa_ml);
-        pa_ml=NULL;
-        return;
-    }
 
     pa_threaded_mainloop_unlock(pa_ml);
 
@@ -790,27 +758,15 @@ void quisk_start_sound_pulseaudio(struct sound_dev **pCapture, struct sound_dev
             num_pa_devices++;
     }
 
-    if (verbose)
+    if (quisk_sound_state.verbose_pulse)
         printf("Waiting for %d streams to start\n", num_pa_devices);
     
-    while (streams_ready < num_pa_devices); // wait for all the devices to open
+    while (streams_to_start < num_pa_devices); // wait for all the devices to open
 
-    if (verbose)
+    if (quisk_sound_state.verbose_pulse)
         printf("All streams started\n");
+    
 
-    if (pa_IQ_ctx && verbose == 2) {
-        if (!(time_event_IQ = pa_context_rttime_new(pa_IQ_ctx, pa_rtclock_now() + TIME_EVENT_USEC,
-                                                    time_event_callback, RemotePulseDevices)))
-            printf("remote pa_context_rttime_new() failed.");
-    }
-
-    if (pa_ctx && verbose == 2) {
-        if (!(time_event_local = pa_context_rttime_new(pa_ctx, pa_rtclock_now() + TIME_EVENT_USEC,
-                                                       time_event_callback, LocalPulseDevices))) {
-            printf("local pa_context_rttime_new() failed.");
-            exit(1);
-        }
-    }
 }
 
 
@@ -818,26 +774,18 @@ void quisk_start_sound_pulseaudio(struct sound_dev **pCapture, struct sound_dev
 void quisk_close_sound_pulseaudio() {
     int i = 0;
     
-    if (verbose)
+    if (quisk_sound_state.verbose_pulse)
         printf("Closing Pulseaudio interfaces \n ");
 
     while (OpenPulseDevices[i]) {
         pa_stream_disconnect(OpenPulseDevices[i]);
         pa_stream_unref(OpenPulseDevices[i]);
+        OpenPulseDevices[i] = '\0';
         i++;
     }
+    
 
-    if (time_event_local) {
-        assert(pa_mlapi);
-        pa_mlapi->time_free(time_event_local);
-    }
-
-    if (time_event_IQ) {
-        assert(pa_mlapi);
-        pa_mlapi->time_free(time_event_IQ);
-    }
-
-    if (verbose)
+    if (quisk_sound_state.verbose_pulse)
         printf("Waiting for %d streams to disconnect\n", streams_ready);
 
     while(streams_ready > 0);
@@ -845,16 +793,19 @@ void quisk_close_sound_pulseaudio() {
     if (pa_IQ_ctx) {
         pa_context_disconnect(pa_IQ_ctx);
         pa_context_unref(pa_IQ_ctx);
+	pa_IQ_ctx = NULL;
     }
 
     if (pa_ctx) {
         pa_context_disconnect(pa_ctx);
         pa_context_unref(pa_ctx);
+	pa_ctx = NULL;
     }
 
     if (pa_ml) {
         pa_threaded_mainloop_stop(pa_ml);
         pa_threaded_mainloop_free(pa_ml);
+	pa_ml = NULL;
     }
 }
 
@@ -919,6 +870,10 @@ static void pa_sinklist_cb(pa_context *c, const pa_sink_info *l, int eol, void *
     if (eol > 0)	// If eol is set to a positive number, you're at the end of the list
 		return;
     source_sink(l->name, l->description, l->proplist, (PyObject *)userdata);
+    if ( ! strcmp(l->name, "QuiskDigitalInput"))
+        have_QuiskDigitalInput = 1;
+    if ( ! strcmp(l->name, "QuiskDigitalOutput"))
+        have_QuiskDigitalOutput = 1;
 }
 
 static void pa_sourcelist_cb(pa_context *c, const pa_source_info *l, int eol, void *userdata) {
@@ -927,6 +882,10 @@ static void pa_sourcelist_cb(pa_context *c, const pa_source_info *l, int eol, vo
     source_sink(l->name, l->description, l->proplist, (PyObject *)userdata);
 }
 
+static void index_callback(pa_context *c, uint32_t idx, void *userdata) {
+    //printf("%u\n", idx);
+}
+
 PyObject * quisk_pa_sound_devices(PyObject * self, PyObject * args)
 {	// Return a list of PulseAudio device names [pycapt, pyplay]
 	PyObject * pylist, * pycapt, * pyplay;
@@ -954,7 +913,7 @@ PyObject * quisk_pa_sound_devices(PyObject * self, PyObject * args)
  
 	// This function connects to the pulse server
 	if (pa_context_connect(pa_names_ctx, NULL, 0, NULL) < 0) {
-	   if (verbose)
+	   if (quisk_sound_state.verbose_pulse)
 	      printf("No local daemon to connect to for show_pulse_audio_devices option\n");
 	   return pylist;
 	}
@@ -992,6 +951,45 @@ PyObject * quisk_pa_sound_devices(PyObject * self, PyObject * args)
 			pa_mainloop_iterate(pa_names_ml, 1, NULL);
 			break;
 		case 3:
+			if (pa_operation_get_state(pa_op) == PA_OPERATION_DONE) {
+				pa_operation_unref(pa_op);
+				state = 4;
+			}
+			else
+				pa_mainloop_iterate(pa_names_ml, 1, NULL);
+			break;
+		// The following loads modules for digital input and output if they are not present.
+		case 4:
+			if ( ! have_QuiskDigitalInput) {
+				pa_op = pa_context_load_module(pa_names_ctx, "module-null-sink",
+					"sink_name=QuiskDigitalInput sink_properties=device.description=QuiskDigitalInput",
+					index_callback, NULL);
+				state = 5;
+				pa_mainloop_iterate(pa_names_ml, 1, NULL);
+			}
+			else
+				state = 6;
+			break;
+		case 5:
+			if (pa_operation_get_state(pa_op) == PA_OPERATION_DONE) {
+				pa_operation_unref(pa_op);
+				state = 6;
+			}
+			else
+				pa_mainloop_iterate(pa_names_ml, 1, NULL);
+			break;
+		case 6:
+			if ( ! have_QuiskDigitalOutput) {
+				pa_op = pa_context_load_module(pa_names_ctx, "module-null-sink",
+					"sink_name=QuiskDigitalOutput sink_properties=device.description=QuiskDigitalOutput",
+					index_callback, NULL);
+				state = 7;
+				pa_mainloop_iterate(pa_names_ml, 1, NULL);
+			}
+			else
+				state = 9;
+			break;
+		case 7:
 			if (pa_operation_get_state(pa_op) == PA_OPERATION_DONE) {
 				pa_operation_unref(pa_op);
 				state = 9;
